from pathlib import Path
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.core.exceptions import ImproperlyConfigured
from html2text import html2text


class HTMLEmailTemplate(object):
    subject = ''
    template_name = None

    def __init__(self, from_email=None, to=None, bcc=None, attachments=None, headers=None,
                 alternatives=None, cc=None, reply_to=None, context=None):
        self.from_email = from_email
        self.to = to
        self.bcc = bcc
        if attachments:
            self.attachments = attachments
        else:
            self.attachments = []
        self.headers = headers
        self.alternatives = alternatives
        self.cc = cc
        self.reply_to = reply_to
        self.context = context

    def get_subject(self):
        return self.subject

    def get_template_name(self):
        if self.template_name is None:
            raise ImproperlyConfigured(
                "HTMLMessage requires a definitions of 'template_name'"
                "or an implementation of 'get_template_name()'"
            )
        return self.template_name

    def attach_file(self, path, mimetype=None):
        path = Path(path)
        with path.open('rb') as f:
            content = f.read()
            self.attachments.append((path.name, content, mimetype))

    def get_html_body(self):
        return render_to_string(self.get_template_name(), self.context)

    def get_mail(self):
        html_body = self.get_html_body()
        mail = EmailMultiAlternatives(
            subject=self.get_subject(),
            body=html2text(html_body),
            from_email=self.from_email,
            to=self.to,
            attachments=self.attachments
        )
        mail.attach_alternative(html_body, 'text/html')
        return mail

    def send(self, fail_silently=False):
        return self.get_mail().send(fail_silently=fail_silently)




import os
from django.test import TestCase, override_settings
from django.core import mail
from django.conf import settings
from email_views.emails import HTMLEmailTemplate


class TestMessage(HTMLEmailTemplate):
    subject = 'test subject'
    template_name = 'dummy.html'


class TestCore(TestCase):
    def setUp(self):
        pass

    def test_sending(self):
        message = TestMessage(from_email='test@domain.tld', to=['to@domain.tld'], context={'var': 'world'})
        result = message.send()
        self.assertEqual(result, 1)
        self.assertEqual(len(mail.outbox), 1)
        email_message = mail.outbox[0]
        self.assertEqual(email_message.subject, 'test subject')
        self.assertEqual(len(email_message.alternatives), 1)
        self.assertEqual(email_message.alternatives[0][1], 'text/html')
        html = email_message.alternatives[0][0]
        self.assertEqual(html, 'hello_world')
        self.assertEqual(email_message.body, 'hello_world\n\n')
        self.assertEqual(email_message.to, ['to@domain.tld'])
        self.assertEqual(email_message.from_email, 'test@domain.tld')

    def test_attach_file(self):
        message = TestMessage(to=['to@domain.tld'])
        message.attach_file(os.path.join(settings.BASE_DIR, 'tests/templates/dummy_attachment'), 'text/plain')
        message.send()
        self.assertEqual(len(mail.outbox[0].attachments), 1)
        attachment = mail.outbox[0].attachments[0]
        self.assertEqual(attachment[0], 'dummy_attachment')
        self.assertEqual(attachment[1], 'dummy text')
        self.assertEqual(attachment[2], 'text/plain')



import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = "38dh*skf8sjfhs287dh&^hd8&3hdg*j2&sd"
DEBUG = False

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'
    }
}

EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'

INSTALLED_APPS = [

]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'tests/templates')],
        'APP_DIRS': False,
    }
]
